from math import sqrt, ceil, floor
from typing import List
import matplotlib.pyplot as plt

def convert_line_to_numbers(rec):
    return (int(x.strip()) for x in rec.strip().split(","))

class Stat:
    label: int
    records: List[int]

    def __init__(self, label, records):
        self.label = label
        self.records = records

    @property
    def mean(self):
        return sum(self.records)/len(self.records)

    @property
    def sd(self):
        mu = self.mean
        return sqrt(sum([((r - mu) ** 2) for r in self.records])/len(self.records))

    def __repr__(self):
        return f"{self.label}: μ={self.mean}, σ={self.sd}"

def convert_file_to_stat(name):
    lines = []
    with open(name, "r") as fp:
        lines = fp.readlines()
    m = dict()
    for (order, res) in ([convert_line_to_numbers(line) for line in lines]):
        if order not in m.keys():
            m[order] = [res]
        else:
            m[order].append(res)
    stats = []
    for key in m.keys():
        l = m[key]
        stats.append(Stat(key, l))
    return stats

iterate_1 = convert_file_to_stat("iterate_1")
iterate_2 = convert_file_to_stat("iterate_2")

print("iterate_1:")
for x in iterate_1:
    print(x)
print("\niterate_2:")
for x in iterate_2:
    print(x)
print("")

orders = [x.label for x in iterate_1]
y_1 = [x.mean for x in iterate_1]
e_1 = [x.sd for x in iterate_1]
y_2 = [x.mean for x in iterate_2]
e_2 = [x.sd for x in iterate_2]

# plt.plot(orders, y_1, label="iterate_1")
# plt.plot(orders, y_2, label="iterate_2")
plt.errorbar(orders, y_1, e_1, label="byte-by-byte")
plt.errorbar(orders, y_2, e_2, label="word-by-word")
plt.xlabel("Orders of magnitude (2^n)")
plt.ylabel("Time taken (msec)")
plt.legend()
plt.show()
